' Add a Reference to System.DirectoryServices.dll
' Add a Reference to System.Management
' Add a Reference to System.Data.OracleClient
' Add a COM Reference to Windows Script Host Object Model
Imports System.IO
Imports System.Threading
Imports System.ComponentModel
Imports System.Collections.Specialized

'
' A console version of SOSOS for unattended polling.  This program is designed
' to operate as a scheduled task and therefore has no user interface. It takes
' the defaults from both the config file settings and optional command-line
' arguments. 
'
Module Main

    Private Enum PollLogLevels As Integer
        None = 0
        Summary
        Errors
        Full
    End Enum

    Private Enum PollMethods As Integer
        AD = 0
        NT
        Browse
        File
        Age
        IP
    End Enum

    Private tw As TextWriter
    Private fs As FileStream
    Private TotalCount, SuccessCount As Integer
    Private UsePingValidation As Boolean
    Private PollLogLevel As PollLogLevels
    Private PollTimeOut As Integer

    Dim WithEvents BackgroundWorker1 As New BackgroundWorker

    '
    ' Since this is a command-line program that's intended to be run as a
    ' scheduled task, we briefly display error messages in the command window
    ' and then write them to the event log.  We also attempt to have a 
    ' meaningful exit code for the task scheduler to record.
    '
    Sub Main()
        Dim arg, nextarg, PC_List() As String
        Dim asm As System.Reflection.Assembly
        Dim i, maxThreads, portThreads As Integer
        Dim Domain, OU_Filter, NameFilter, PollLogDirectory, PollByFileFile, PollLogFile As String
        Dim PoolSize, PollByAge As Integer
        Dim PollMethod As PollMethods
        Dim evt As EventLog

        Try
            asm = System.Reflection.Assembly.GetExecutingAssembly
            Console.WriteLine(asm.GetName.Name & " v" & asm.GetName.Version.ToString & " (" & asm.ImageRuntimeVersion & ")")
            Console.WriteLine("Starting: " & Now)

            ' Let's check out the necessary assembly permissions now, so that we 
            ' don't have any surprises later. 
            Dim Testit As New TestPerm
            If Testit.DoTestPerm = False Then
                ErrorMessage(".Net Framework Code Access Permissions not set")
                Environment.ExitCode = 12 'ERROR_INVALID_ACCESS
                Exit Sub
            End If

            ' Let's start with the default settings... the command line
            ' arguments below may override these values.
            UsePingValidation = My.Settings.UsePingValidation
            Domain = My.Settings.Domain
            OU_Filter = My.Settings.OU_Filter
            NameFilter = My.Settings.NameFilter
            PoolSize = CInt(My.Settings.PoolSize)
            PollLogDirectory = My.Settings.PollLogDirectory.TrimEnd("\"c)
            PollLogFile = My.Settings.PollLogFile
            PollLogLevel = CType([Enum].Parse(GetType(PollLogLevels), My.Settings.PollLogLevel), PollLogLevels)
            PollByAge = CInt(My.Settings.PollByAge)
            PollByFileFile = My.Settings.PollByFileFile
            PollMethod = CType([Enum].Parse(GetType(PollMethods), My.Settings.PollMethod), PollMethods)
            PollTimeOut = CInt(My.Settings.PollTimeout)

            ' parse the command line
            Try
                For i = 0 To My.Application.CommandLineArgs.Count - 1
                    arg = My.Application.CommandLineArgs(i)
                    nextarg = ""
                    If i < My.Application.CommandLineArgs.Count - 1 Then
                        nextarg = My.Application.CommandLineArgs(i + 1)
                        If nextarg.StartsWith("/") Then
                            Throw New ApplicationException("missing argument at " & nextarg)
                        End If
                    End If
                    Select Case (arg)
                        Case "/UsePingValidation"
                            UsePingValidation = CBool(nextarg)
                        Case "/Domain"
                            Domain = nextarg
                        Case "/OU_Filter"
                            OU_Filter = nextarg
                        Case "/NameFilter"
                            NameFilter = nextarg
                        Case "/PoolSize"
                            PoolSize = CInt(nextarg)
                        Case "/PollLogDirectory"
                            PollLogDirectory = nextarg.TrimEnd("\"c)
                        Case "/PollLogFile"
                            PollLogFile = nextarg
                        Case "/PollLogLevel"
                            PollLogLevel = CType([Enum].Parse(GetType(PollLogLevels), nextarg), PollLogLevels)
                        Case "/PollByAge"
                            PollByAge = CInt(nextarg)
                        Case "/PollByFileFile"
                            PollByFileFile = nextarg
                        Case "/PollMethod"
                            PollMethod = CType([Enum].Parse(GetType(PollMethods), nextarg), PollMethods)
                        Case "/PollTimeout"
                            PollTimeOut = CInt(nextarg)
                        Case Else
                            Throw New ApplicationException("invalid command-line token " & arg)
                    End Select
                    i += 1
                Next
            Catch ex As Exception
                ErrorMessage("Bad command line arguments" & vbCrLf & ex.Message)
                Environment.ExitCode = 87 'ERROR_INVALID_PARAMETER
                Exit Sub
            End Try

            ' Open the logfile
            If PollLogLevel > PollLogLevels.None Then
                Try
                    ' automatically create the logfile name?
                    If PollLogFile = "" Then
                        If PollLogDirectory = "" Then
                            PollLogDirectory = Environment.ExpandEnvironmentVariables("%TEMP%")
                        End If
                        PollLogFile = PollLogDirectory & "\PollSOSOS_" & Now.ToString("yyyyMMdd") & ".txt"
                    End If

                    fs = New FileStream(PollLogFile, FileMode.Append)
                    ' use a thread-safe stream writer
                    tw = TextWriter.Synchronized(New StreamWriter(fs))

                    asm = System.Reflection.Assembly.GetExecutingAssembly
                    tw.WriteLine(asm.GetName.Name & " v" & asm.GetName.Version.ToString & " (" & asm.ImageRuntimeVersion & ")")
                    tw.WriteLine("Started: " & Now)
                Catch ex As Exception
                    ErrorMessage("Can't open logfile" & vbCrLf & ex.Message)
                    Environment.ExitCode = 2 ' ERROR_FILE_NOT_FOUND
                    Exit Sub
                End Try
            End If

            ' the next order of business is to create the list of PCs that we
            ' want to poll. 
            Console.WriteLine("Getting network list...")
            PC_List = Nothing
            Try
                Select Case PollMethod
                    Case PollMethods.AD
                        Dim net As New GetNetList

                        PC_List = net.GetPCOUList(Domain, OU_Filter, NameFilter, False)
                    Case PollMethods.Age
                        Dim marker As Date
                        Dim AgeQuery As String
                        Dim db As New Database

                        marker = Now.AddDays(-PollByAge)

                        ' There is no universal way to format date constants.
                        ' Paramertized queries aren't much help because they
                        ' too are not "agnostic".  We also can't really make
                        ' this culture aware because we don't know the 
                        ' culture settings of the database.  Oh well...
                        If My.Settings.ConnectionString.Contains(".Jet.") Then
                            AgeQuery = "Select PC_Name from SOS where CurrentDate < #" & marker.ToString & "#"
                        Else
                            If My.Settings.DatabaseProvider = "System.Data.OracleClient" Or My.Settings.ConnectionString.Contains("MSDAORA") Then
                                AgeQuery = "Select PC_Name from SOS where CurrentDate < to_date('" & marker.ToString & "', 'MM/DD/YYYY HH:MI:SS AM')"
                            Else
                                AgeQuery = "Select PC_Name from SOS where CurrentDate < '" & marker.ToString & "'"
                            End If
                        End If
                        PC_List = db.GetPCList(AgeQuery)
                    Case PollMethods.Browse
                        Dim net As New GetNetList

                        PC_List = net.GetPCBrowseList(Domain, NameFilter)
                    Case PollMethods.File
                        Dim sr As StreamReader
                        Dim sc As New StringCollection
                        Dim temp As String

                        If [File].Exists(PollByFileFile) Then
                            sr = [File].OpenText(PollByFileFile)
                            Do While sr.Peek() > -1
                                temp = sr.ReadLine
                                ' skip blank lines and comment
                                If temp <> "" And Not temp.StartsWith(";") Then
                                    sc.Add(temp)
                                End If
                            Loop
                            sr.Close()
                        End If

                        If sc.Count > 0 Then
                            ReDim PC_List(sc.Count - 1)
                            sc.CopyTo(PC_List, 0)
                        End If
                    Case PollMethods.NT
                        Dim net As New GetNetList

                        PC_List = net.GetPCDomList(Domain, NameFilter)
                    Case Else
                        Throw New ApplicationException("Poll Method " & PollMethod.ToString & " is not supported")
                End Select
            Catch ex As Exception
                If Not IsNothing(tw) And PollLogLevel >= PollLogLevels.Errors Then
                    tw.WriteLine("Error getting list of PCs, " & ex.Message.TrimEnd(vbCrLf.ToCharArray))
                    tw.WriteLine("Finished: " & Now)
                    tw.Close()
                    fs.Close()
                End If

                ErrorMessage("Error getting list of PCs " & vbCrLf & ex.Message)
                Environment.ExitCode = 59 'ERROR_UNEXP_NET_ERR
                Exit Sub
            End Try

            ' a bit of sanity checking (not really an error...)
            If IsNothing(PC_List) OrElse PC_List.Length = 0 Then
                evt = New EventLog

                If Not IsNothing(tw) And PollLogLevel >= PollLogLevels.Summary Then
                    tw.WriteLine("No PCs in the list")
                    tw.WriteLine("Finished: " & Now)
                    tw.Close()
                    fs.Close()
                End If

                Console.WriteLine("No PCs in the list")
                Console.WriteLine("Finished: " & Now)
                evt.Source = "PollSOSOS"
                evt.WriteEntry("No PCs in the list", System.Diagnostics.EventLogEntryType.Warning)
                System.Threading.Thread.Sleep(1000)
                Exit Sub
            End If

            Console.WriteLine("Started polling " & PC_List.Length & " PCs")

            ' set the maxiumum number of threads in the pool
            ThreadPool.GetMaxThreads(maxThreads, portThreads)
            ThreadPool.SetMaxThreads(PoolSize, portThreads)

            ' Start up spooling of worker treads.  This might seem like a bit
            ' of overkill, since this is just a console application.  However,
            ' folks don't like CMD windows to appear frozen.
            BackgroundWorker1.RunWorkerAsync(PC_List)

            ' a crude way of waiting...
            Console.ReadLine()
        Catch ex As Exception
            ' Catch any other totally unexpected errors
            ErrorMessage("Internal Error" & vbCrLf & ex.Message & vbCrLf & ex.StackTrace)
            Environment.ExitCode = 1287 ' ERROR_UNIDENTIFIED_ERROR
        End Try

    End Sub

    '
    ' Just a convenient way to package up all of the data that we need to
    ' send to the delegate
    '
    Public Class package
        Public waitHandle As AutoResetEvent
        Public RemotePC As String
    End Class

    '
    ' Manages the work queue.  It puts the items into the queue, waits
    ' for them to complete, and then closes out the application.
    '
    Private Sub BackgroundWorker1_DoWork(ByVal sender As Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundWorker1.DoWork
        Dim PC_Name, PC_List() As String
        Dim p As package
        Dim i As Integer
        Dim wh() As AutoResetEvent
        Dim evt As EventLog

        i = 0
        PC_List = CType(e.Argument, String())
        ReDim wh(PC_List.Length - 1)

        ' Start up spooling of worker treads
        For Each PC_Name In PC_List
            wh(i) = New AutoResetEvent(False)

            ' Setup the "package" that contains the information
            ' we need to send to the delegate
            p = New package
            p.waitHandle = wh(i)
            p.RemotePC = PC_Name

            ' put it in the queue
            ThreadPool.QueueUserWorkItem(AddressOf StartIt, p)
            i += 1
        Next

        For i = 0 To UBound(wh)
            ' wait until each thread has signaled that it's done
            ' (or until an hour has passed)
            wh(i).WaitOne(60 * 60 * 1000, False)
        Next

        '
        ' When we get this far, all of the scans have been completed, so write
        ' it to the poll log, the console, and to the event log
        '
        If Not IsNothing(tw) Then
            tw.Flush()
            tw.WriteLine("Finished: " & Now())
            tw.WriteLine("Successfully scanned " & SuccessCount & " out of " & TotalCount & " PCs")
            tw.WriteLine("")
            tw.Close()
            fs.Close()
            tw = Nothing
        End If

        Console.WriteLine("")
        Console.WriteLine("Finished: " & Now())
        Console.WriteLine("Successfully scanned " & SuccessCount & " out of " & TotalCount & " PCs")

        evt = New EventLog
        evt.Source = "PollSOSOS"
        evt.WriteEntry("Successfully scanned " & SuccessCount & " out of " & TotalCount & " PCs", System.Diagnostics.EventLogEntryType.Information)

        System.Threading.Thread.Sleep(1000)

        ' The main thread is holding, so we forcably exit the application
        Environment.Exit(0)
    End Sub

    '
    ' Start the polling in such a way that allows us to terminate the
    ' thread if it runs too long. Has the signature of a 
    ' System.Threading.WaitCallback delegate (which is why we had to bundle
    ' all the data it needs into a "package" class)
    '
    Sub StartIt(ByVal State As Object)
        Dim p As package
        Dim wait As Integer

        p = DirectCast(State, package)

        Dim t As New Thread(AddressOf Doit)
        t.Start(State)

        ' not smart, but it is supported
        If PollTimeOut = 0 Then
            wait = -1
        Else
            wait = PollTimeOut * 60 * 1000
        End If

        ' Wait until the thread ends.  But if it takes too long, abort it
        If t.Join(wait) = False Then
            t.Abort()
            ' give it a minute to close down...
            t.Join(60 * 1000)
        End If

        ' signal that we're done
        p.waitHandle.Set()
    End Sub

    ' 
    ' This is where the real work get's done.  We fire up the SOSOS 
    ' components to get the data from the remote PCs.
    '
    Sub Doit(ByVal State As Object)
        Dim ds As New SOSOSDataSet
        Dim snoop As Snoop
        Dim RemotePC As String
        Dim pkg As package

        pkg = DirectCast(State, package)
        RemotePC = pkg.RemotePC
        snoop = Nothing
        Try

            If UsePingValidation = True Then
                Dim p As New Ping
                If p.Ping(RemotePC) > 1000 Then
                    If Not IsNothing(tw) And PollLogLevel = PollLogLevels.Full Then tw.WriteLine(Now() & " " & RemotePC & " Didn't answer a ping")
                    ' this is not an "error", it's an expected condition
                    Exit Sub
                End If
            End If

            Try
                snoop = New Snoop(RemotePC, ds)
            Catch ex As Exception
                If Not ex.GetType Is GetType(System.Runtime.InteropServices.COMException) OrElse DirectCast(ex, System.Runtime.InteropServices.COMException).ErrorCode <> &H800706BA Then
                    ' don't record "The RPC server is unavailable.", since this
                    ' is not a error... it's an expected condition
                    Misc.ErrorLog(ex, RemotePC, "Initialize", Misc.ErrLogLevel.Errors)
                End If
                If Not IsNothing(tw) And PollLogLevel >= PollLogLevels.Errors Then
                    If PollLogLevel = PollLogLevels.Full Then tw.Write(Now() & " ")
                    If IsNothing(ex.InnerException) Then
                        tw.WriteLine(RemotePC & " Initialize Error: " & ex.Message.TrimEnd(vbCrLf.ToCharArray))
                    Else
                        tw.WriteLine(RemotePC & " Initialize Error: " & ex.Message.TrimEnd(vbCrLf.ToCharArray) & ", " & ex.InnerException.Message.TrimEnd(vbCrLf.ToCharArray))
                    End If
                End If
                Exit Sub
            End Try

            ds.SOS(0).RecordType = "Remote"

            If My.Settings.DoAccounts Then snoop.GetAccounts()
            If My.Settings.DoAdmins Then snoop.GetAdmins()
            If My.Settings.DoAutoUpdate Then snoop.GetAutoUpdate()
            If My.Settings.DoBIOS Then snoop.GetBIOS()
            If My.Settings.DoComponents Then snoop.GetComponents()
            If My.Settings.DoCPU Then snoop.GetCPU()
            If My.Settings.DoDevices Then snoop.GetDevices()
            If My.Settings.DoDisks Then snoop.GetDisks()
            If My.Settings.DoDrives Then snoop.GetDrives()
            If My.Settings.DoEventLogSettings Then snoop.GetEventLogSettings()
            If My.Settings.DoEventLogs Then snoop.GetEventLogs()
            If My.Settings.DoFileInfo Then snoop.GetFileInfo()
            If My.Settings.DoFileInfo Then snoop.GetInstallVer()
            If My.Settings.DoMemory Then snoop.GetMemory()
            If My.Settings.DoModem Then snoop.GetModem()
            If My.Settings.DoMonitor Then snoop.GetMonitor()
            ' SystemInfo goes before Motherboard
            If My.Settings.DoSystemInfo Then snoop.GetSystemInfo()
            If My.Settings.DoMotherboard Then snoop.GetMotherboard()
            If My.Settings.DoMouse Then snoop.GetMouse()
            If My.Settings.DoNetAdapter Then snoop.GetNetAdapter()
            If My.Settings.DoNetConfig Then snoop.GetNetConfig()
            If My.Settings.DoOS Then snoop.GetOS()
            If My.Settings.DoPC Then snoop.GetPC()
            If My.Settings.DoPorts Then snoop.GetPorts()
            If My.Settings.DoPrinters Then snoop.GetPrinters()
            If My.Settings.DoProcesses Then snoop.GetProcesses()
            If My.Settings.DoQFE Then snoop.GetQFE()
            If My.Settings.DoRegistry Then snoop.GetRegistry()
            If My.Settings.DoServices Then snoop.GetServices()
            If My.Settings.DoShares Then snoop.GetShares()
            ' Permissions goes after Shares
            If My.Settings.DoPermissions Then snoop.GetPermissions()
            If My.Settings.DoSoftware Then snoop.GetSoftware()
            If My.Settings.DoSoftware Then snoop.GetSoftwareSummary()
            If My.Settings.DoSystemDrivers Then snoop.GetSystemDrivers()
            If My.Settings.DoTasks Then snoop.GetTasks()
            ' skip UserLogin completely
            If My.Settings.DoVideo Then snoop.GetVideo()
            If My.Settings.DoVirus Then snoop.GetVirus()
            If My.Settings.DoVirus Then snoop.GetVirusSummary()
            ' the user profile section...
            If My.Settings.DoProfiles Then snoop.GetProfiles()
            If My.Settings.DoDesktop Then snoop.GetDesktop()
            If My.Settings.DoEmail Then snoop.GetEmail()
            If My.Settings.DoInternet Then snoop.GetInternet()
            If My.Settings.DoMapped Then snoop.GetMapped()
            If My.Settings.DoStartup Then snoop.GetStartup()

            Try
                Dim db As New Database
                db.RecordData(ds, RemotePC)
                ' these errors are automatically recorded in the ErrorLog
            Catch ex As Exception
                If Not IsNothing(tw) And PollLogLevel >= PollLogLevels.Errors Then
                    If PollLogLevel = PollLogLevels.Full Then tw.Write(Now() & " ")
                    If IsNothing(ex.InnerException) Then
                        tw.WriteLine(RemotePC & " Database Error: " & ex.Message.TrimEnd(vbCrLf.ToCharArray))
                    Else
                        tw.WriteLine(RemotePC & " Database Error: " & ex.Message.TrimEnd(vbCrLf.ToCharArray) & ", " & ex.InnerException.Message.TrimEnd(vbCrLf.ToCharArray))
                    End If
                End If
                Exit Sub
            End Try

            If Not IsNothing(tw) And PollLogLevel = PollLogLevels.Full Then tw.WriteLine(Now & " " & RemotePC)
            SuccessCount = Interlocked.Increment(SuccessCount)
        Catch ex_abort As ThreadAbortException
            ' did we have to abort this thread for taking too long?
            If Not IsNothing(tw) And PollLogLevel >= PollLogLevels.Errors Then
                If PollLogLevel = PollLogLevels.Full Then tw.Write(Now() & " ")
                tw.WriteLine(RemotePC & " Exceeded PollTimeOut limit")
            End If
        Catch ex As Exception
            Misc.ErrorLog(ex, RemotePC, "Internal", Misc.ErrLogLevel.Errors)
            If Not IsNothing(tw) And PollLogLevel >= PollLogLevels.Errors Then
                If PollLogLevel = PollLogLevels.Full Then tw.Write(Now() & " ")
                If IsNothing(ex.InnerException) Then
                    tw.WriteLine(RemotePC & " Internal Error: " & ex.Message.TrimEnd(vbCrLf.ToCharArray))
                Else
                    tw.WriteLine(RemotePC & " Internal Error: " & ex.Message.TrimEnd(vbCrLf.ToCharArray) & ", " & ex.InnerException.Message.TrimEnd(vbCrLf.ToCharArray))
                End If
            End If
        Finally
            Console.Write(".")
            TotalCount = Interlocked.Increment(TotalCount)

            ' requires an explicit Dispose to unmount the registry hives
            If Not IsNothing(snoop) Then
                snoop.Dispose()
            End If
        End Try
    End Sub

    '
    ' Display an error message and write the message to the eventlog
    '
    Private Sub ErrorMessage(ByVal message As String)
        Dim evt As New EventLog

        Beep()
        Console.WriteLine("Error: " & message)
        evt.Source = "PollSOSOS"
        evt.WriteEntry(message, System.Diagnostics.EventLogEntryType.Error)
        System.Threading.Thread.Sleep(5000)
    End Sub
End Module
