' Add a Reference to System.Drawing.dll
' Add a Reference to System.Windows.Form.dll
Imports System.Threading.Thread

'
' A console version of SOSOS for unattended gathering of data.  It is 
' intended to be used in a Login script or in a scheduled task.
' Has the following command-line arguments:
'   /q (quiet) run with no output to the console
'   /w (wallpaper) put a summary "watermark" on the user's desktop
'   /l (login) also record information for the UserLogin table
'   /s (server) run from a scheduled task (records to event log)
'   /wlo (Wallpaper & Login Only)
'   /auto Switch to WLO_mode if not an administrator
'
Module Main

    Sub Main()
        Dim ds As SOSOSDataSet
        Dim snoop As Snoop
        Dim db As Database
        Dim arg, temp, RemotePC As String
        Dim asm As System.Reflection.Assembly
        Dim QuietMode, ServerMode, DoWallPaper, LoginMode, WLO_Mode As Boolean
        Dim evt As EventLog

        RemotePC = ""
        snoop = Nothing
        Try
            ' parse the command line
            QuietMode = False
            ServerMode = False
            DoWallPaper = False
            LoginMode = False
            WLO_Mode = False

            For Each arg In My.Application.CommandLineArgs
                For Each temp In arg.Split("/"c)
                    Select Case temp.ToLower
                        Case "q"
                            QuietMode = True
                        Case "w"
                            DoWallPaper = True
                        Case "l"
                            LoginMode = True
                        Case "s"
                            ServerMode = True
                        Case "wlo"
                            WLO_Mode = True
                            LoginMode = True
                            DoWallPaper = True
                        Case "auto"
                            If Not Misc.IsAdmin Then
                                WLO_Mode = True
                                LoginMode = True
                                DoWallPaper = True
                            End If
                    End Select
                Next
            Next

            ' Let's check out the necessary assembly permissions now, so that we 
            ' don't have any surprises later. 
            Dim Testit As New TestPerm
            If Testit.DoTestPerm = False Then
                If ServerMode Then
                    evt = New EventLog
                    evt.Source = "RunSOSOS"
                    evt.WriteEntry(".Net Framework Code Access Permissions not set", System.Diagnostics.EventLogEntryType.Error)
                End If
                If Not QuietMode Then
                    Beep()
                    Console.WriteLine("RunSOSOS requires significant permissions for it to work properly" & vbCrLf & "Ask your administrator to consult the SOSOS documentation and" & vbCrLf & "adjust the 'Assembly Permissions' or 'Security Zone'")
                    Sleep(5000)
                End If
                Environment.ExitCode = 12 'ERROR_INVALID_ACCESS
                Exit Sub
            End If

            If Not QuietMode Then
                asm = System.Reflection.Assembly.GetExecutingAssembly
                Console.WriteLine(asm.GetName.Name & " v" & asm.GetName.Version.ToString & " (" & asm.ImageRuntimeVersion & ")")
                Console.WriteLine("Starting: " & Now)
            End If

            ' check if the last thing on the command line looks like a PC name
            If My.Application.CommandLineArgs.Count > 0 Then
                temp = My.Application.CommandLineArgs(My.Application.CommandLineArgs.Count - 1)
                If temp <> "" And Not temp.StartsWith("/") Then
                    RemotePC = temp

                    If Not QuietMode Then
                        Console.WriteLine("Remote: " & RemotePC)
                    End If

                    ' let's make sure the remote PC is really there...
                    If My.Settings.UsePingValidation Then
                        Dim p As New Ping
                        If p.Ping(RemotePC) > 1000 Then
                            If ServerMode Then
                                evt = New EventLog
                                evt.Source = "RunSOSOS"
                                evt.WriteEntry(RemotePC & " didn't answer a ping", System.Diagnostics.EventLogEntryType.Warning)
                            End If
                            If Not QuietMode Then
                                Beep()
                                Console.WriteLine(RemotePC & " didn't answer a ping")
                                Sleep(5000)
                            End If
                            Environment.ExitCode = 1232 'ERROR_HOST_UNREACHABLE
                            Exit Sub
                        End If
                    End If
                End If
            End If

            If Not QuietMode Then
                Console.WriteLine("Gathering data")
                Console.Write(".")
            End If

            ' start it up...
            Try
                ds = New SOSOSDataSet
                snoop = New Snoop(RemotePC, ds)
            Catch ex As Exception
                If Not ex.GetType Is GetType(System.Runtime.InteropServices.COMException) OrElse DirectCast(ex, System.Runtime.InteropServices.COMException).ErrorCode <> &H800706BA Then
                    ' don't record "The RPC server is unavailable.", since this
                    ' is not a error... it's an expected condition
                    Misc.ErrorLog(ex, RemotePC, "Initialize", Misc.ErrLogLevel.Errors)
                End If
                If ServerMode Then
                    evt = New EventLog
                    evt.Source = "RunSOSOS"
                    evt.WriteEntry("Error getting initial data" & vbCr & ex.Message, System.Diagnostics.EventLogEntryType.Error)
                End If
                If Not QuietMode Then
                    Beep()
                    Console.WriteLine("Error getting initial data")
                    Sleep(5000)
                End If
                Environment.ExitCode = 1627 'ERROR_FUNCTION_FAILED
                Exit Sub
            End Try

            ' there are 4 choices here for the RecordType
            If LoginMode Then
                ds.SOS(0).RecordType = "Login"
            Else
                ds.SOS(0).RecordType = "Manual"
            End If
            If RemotePC <> "" Then
                ds.SOS(0).RecordType = "Remote"
            End If
            If ServerMode Then
                ds.SOS(0).RecordType = "Scheduled"
            End If

            ' gather the data...
            If My.Settings.DoAccounts And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetAccounts()
            End If

            If My.Settings.DoAdmins And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetAdmins()
            End If

            If My.Settings.DoAutoUpdate Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetAutoUpdate()
            End If

            If My.Settings.DoBIOS Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetBIOS()
            End If

            If My.Settings.DoComponents And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetComponents()
            End If

            If My.Settings.DoCPU Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetCPU()
            End If

            If My.Settings.DoDevices And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetDevices()
            End If

            If My.Settings.DoDisks And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetDisks()
            End If

            If My.Settings.DoDrives Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetDrives()
            End If

            If My.Settings.DoEventLogSettings And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetEventLogSettings()
            End If

            If My.Settings.DoEventLogs And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetEventLogs()
            End If

            If My.Settings.DoFileInfo Then
                If Not QuietMode Then Console.Write(".")
                If Not WLO_Mode Then
                    snoop.GetFileInfo()
                End If
                snoop.GetInstallVer()
            End If

            If My.Settings.DoMemory Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetMemory()
            End If

            If My.Settings.DoModem And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetModem()
            End If

            If My.Settings.DoMonitor And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetMonitor()
            End If

            ' should be before GetMotherboard
            If My.Settings.DoSystemInfo Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetSystemInfo()
            End If

            If My.Settings.DoMotherboard And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetMotherboard()
            End If

            If My.Settings.DoMouse And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetMouse()
            End If

            If My.Settings.DoNetAdapter Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetNetAdapter()
            End If

            If My.Settings.DoNetConfig Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetNetConfig()
            End If

            If My.Settings.DoOS Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetOS()
            End If

            If My.Settings.DoPC Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetPC()
            End If

            If My.Settings.DoPorts And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetPorts()
            End If

            If My.Settings.DoPrinters And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetPrinters()
            End If

            If My.Settings.DoProcesses And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetProcesses()
            End If

            If My.Settings.DoQFE Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetQFE()
            End If

            If My.Settings.DoRegistry And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetRegistry()
            End If

            If My.Settings.DoServices And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetServices()
            End If

            If My.Settings.DoShares And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetShares()
            End If

            ' should be after GetShares
            If My.Settings.DoPermissions And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetPermissions()
            End If

            If My.Settings.DoSoftware Then
                If Not QuietMode Then Console.Write(".")
                If Not WLO_Mode Then
                    snoop.GetSoftware()
                End If
                snoop.GetSoftwareSummary()
            End If

            If My.Settings.DoSystemDrivers And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetSystemDrivers()
            End If

            If My.Settings.DoTasks And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetTasks()
            End If

            ' only if in the "Login mode"
            If My.Settings.DoUserLogin And LoginMode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetUserLogin()
            End If

            If My.Settings.DoVideo And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetVideo()
            End If

            If My.Settings.DoVirus Then
                If Not QuietMode Then Console.Write(".")
                If Not WLO_Mode Then
                    snoop.GetVirus()
                End If
                snoop.GetVirusSummary()
            End If

            ' The user profile section...
            If My.Settings.DoProfiles And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetProfiles()
            End If

            If My.Settings.DoDesktop And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetDesktop()
            End If

            If My.Settings.DoEmail And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetEmail()
            End If

            If My.Settings.DoInternet And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetInternet()
            End If

            If My.Settings.DoMapped And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetMapped()
            End If

            If My.Settings.DoStartup And Not WLO_Mode Then
                If Not QuietMode Then Console.Write(".")
                snoop.GetStartup()
            End If

            If Not QuietMode Then Console.WriteLine("")

            ' Make wallpaper?  You can only create wallpaper with data
            ' from your own PC.
            If DoWallPaper And RemotePC = "" Then
                If Not QuietMode Then Console.WriteLine("Creating Wallpaper")
                Try
                    Dim wp As New WallPaper
                    wp.MakeWallPaper(ds)
                Catch ex As Exception
                    Misc.ErrorLog(ex, RemotePC, "Wallpaper", Misc.ErrLogLevel.Errors)
                    If Not QuietMode Then
                        Beep()
                        Console.WriteLine("Error Creating Wallpaper " & ex.Message)
                        Sleep(5000)
                    End If
                End Try
                ' not fatal
            End If

            ' save the data into the database
            If Not QuietMode Then Console.WriteLine("Saving data")
            Try
                db = New Database()
                If WLO_Mode Then
                    db.UpdateLoginOnly(ds, RemotePC)
                Else
                    db.RecordData(ds, RemotePC)
                End If
            Catch ex As Exception
                If ServerMode Then
                    evt = New EventLog
                    evt.Source = "RunSOSOS"
                    evt.WriteEntry("Error saving the data" & vbCr & ex.Message, System.Diagnostics.EventLogEntryType.Error)
                End If
                If Not QuietMode Then
                    Beep()
                    Console.WriteLine("Error saving the data")
                    Sleep(5000)
                End If
                Environment.ExitCode = 4313 'ERROR_DATABASE_FAILURE
                Exit Sub
            End Try

            If ServerMode Then
                evt = New EventLog
                evt.Source = "RunSOSOS"
                evt.WriteEntry("RunSOSOS completed successfully", System.Diagnostics.EventLogEntryType.Information)
            End If
            If Not QuietMode Then
                Console.WriteLine("Finished: " & Now)
                If RemotePC = "" Then
                    Sleep(1000)
                End If
            End If
        Catch ex As Exception
            ' Catch any other totally unexpected errors
            Try
                Misc.ErrorLog(ex, RemotePC, "Internal", Misc.ErrLogLevel.Errors)
            Catch
            End Try

            If ServerMode Then
                evt = New EventLog
                evt.Source = "RunSOSOS"
                evt.WriteEntry("Internal Error" & vbCr & ex.Message, System.Diagnostics.EventLogEntryType.Error)
            End If
            If Not QuietMode Then
                Beep()
                Console.WriteLine("Error " & ex.Message)
                Sleep(5000)
            End If
            Environment.ExitCode = 1287 ' ERROR_UNIDENTIFIED_ERROR
        Finally
            ' requires an explicit Dispose to unmount the registry hives
            If Not IsNothing(snoop) Then
                snoop.Dispose()
            End If
        End Try
    End Sub

End Module
