' Add a Reference to System.DirectoryServices
Imports System.Runtime.InteropServices
Imports System.ComponentModel
Imports System.DirectoryServices
Imports System.Collections.Specialized

'
' A series of methods to get a list of PCs.  The supported techniques are:
'     Browsing the My Network Places
'     Querying an Active Directory Domain using LDAP
'     Querying a WinNT Domain using ADSI
'
Class GetNetList

#Region "API Region"
    'typedef struct _NETRESOURCE {
    '  DWORD dwScope;
    '  DWORD dwType;
    '  DWORD dwDisplayType;
    '  DWORD dwUsage;
    '  LPTSTR lpLocalName;
    '  LPTSTR lpRemoteName;
    '  LPTSTR lpComment;
    '  LPTSTR lpProvider;
    '} NETRESOURCE;
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi, Pack:=4)> _
    Private Structure NETRESOURCE
        Dim dwScope As Integer
        Dim dwType As Integer
        Dim dwDisplayType As Integer
        Dim dwUsage As Integer
        Dim lpLocalName As String
        Dim lpRemoteName As String
        Dim lpComment As String
        Dim lpProvider As String
    End Structure

    'DWORD WNetOpenEnum(
    '  DWORD dwScope,
    '  DWORD dwType,
    '  DWORD dwUsage,
    '  LPNETRESOURCE lpNetResource,
    '  LPHANDLE lphEnum
    ');
    Private Declare Ansi Function WNetOpenEnum Lib "mpr.dll" Alias "WNetOpenEnumA" ( _
        ByVal dwScope As Integer, _
        ByVal dwType As Integer, _
        ByVal dwUsage As Integer, _
        ByRef lpNetResource As NETRESOURCE, _
        ByRef lphEnum As IntPtr _
    ) As Integer

    'DWORD WNetEnumResource(
    '  HANDLE hEnum,
    '  LPDWORD lpcCount,
    '  LPVOID lpBuffer,
    '  LPDWORD lpBufferSize
    ');
    Private Declare Ansi Function WNetEnumResource Lib "mpr.dll" Alias "WNetEnumResourceA" ( _
        ByVal hEnum As IntPtr, _
        ByRef lpcCount As Integer, _
        ByVal lpBuffer As IntPtr, _
        ByRef lpBufferSize As Integer _
    ) As Integer

    'DWORD WNetCloseEnum(
    '  HANDLE hEnum
    ');
    Private Declare Function WNetCloseEnum Lib "mpr.dll" ( _
        ByVal hEnum As IntPtr _
    ) As Integer

    Private Const RESOURCEDISPLAYTYPE_DOMAIN As Integer = &H1
    Private Const RESOURCEDISPLAYTYPE_NETWORK As Integer = &H6
    Private Const RESOURCE_GLOBALNET As Integer = &H2
    Private Const RESOURCEUSAGE_CONTAINER As Integer = &H2
    Private Const RESOURCETYPE_ANY As Integer = &H0
    Private Const ERROR_NO_MORE_ITEMS As Integer = 259&
#End Region

    '
    ' Return a list of available Domains/Workgroups that can be seen in the
    ' "My Network Places" browse list
    '
    Public Function GetDomainList() As String()
        Dim ans(-1) As String
        Dim sc As New StringCollection

        ' do a "level 1" search of the My Network Places
        GetEnumNet(Nothing, 1, "", sc)

        If sc.Count > 0 Then
            ReDim ans(sc.Count - 1)
            sc.CopyTo(ans, 0)
        End If

        Return ans
    End Function

    '
    ' Get the PCs that you can see from the "My Network Places" browse list
    '
    Function GetPCBrowseList(ByVal DomainName As String, Optional ByVal filter As String = "") As String()
        Dim buf, ans(-1) As String
        Dim sc As New StringCollection
        Dim sc_new As New StringCollection

        ' do a "level 2" search of My Network Places starting using the domain
        ' (or workgroup) as a starting point.
        GetEnumNet(Nothing, 2, DomainName, sc)

        ' filter the list
        filter = filter.ToUpper
        For Each buf In sc
            If filter = "" OrElse (buf = filter Or buf Like filter) Then
                sc_new.Add(buf)
            End If
        Next

        If sc_new.Count > 0 Then
            ReDim ans(sc_new.Count - 1)
            sc_new.CopyTo(ans, 0)
        End If

        Return ans
    End Function

    '
    ' Get the PC list from an NT-style domain using the LDAP WinNT provider
    '
    Function GetPCDomList(ByVal DomainName As String, Optional ByVal filter As String = "") As String()
        Dim dirs, dir As DirectoryEntry
        Dim ans(-1) As String
        Dim sc As New StringCollection

        filter = filter.ToUpper
        dirs = New DirectoryEntry("WinNT://" & DomainName)
        For Each dir In dirs.Children
            ' collect data on all of our Domain PCs
            If dir.SchemaClassName = "Computer" Then
                If filter = "" OrElse dir.Name = filter Or dir.Name Like filter Then
                    ' Add it to the list
                    sc.Add(dir.Name)
                End If
            End If
        Next

        If sc.Count > 0 Then
            ReDim ans(sc.Count - 1)
            sc.CopyTo(ans, 0)
        End If

        Return ans
    End Function

    '
    ' Get the PC list from an Active Directory domain using an LDAP query
    '
    Public Function GetPCOUList(ByVal DomainName As String, ByVal OU_Name As String, ByVal Filter As String, ByVal Svr_Only As Boolean) As String()
        Dim entry, de As DirectoryEntry
        Dim srch As DirectorySearcher
        Dim result As SearchResult
        Dim ldap, buf, os, ans(-1) As String
        Dim sc As New StringCollection

        ldap = "LDAP://" & DomainName & "/" & OU_Name

        ' set our starting place to search
        entry = New DirectoryEntry(ldap)
        srch = New DirectoryServices.DirectorySearcher(entry)
        srch.Filter = "(ObjectCategory=computer)"
        ' A work around for a bug... this shouldn't be needed
        srch.PageSize = 1000

        ' OK, let's do it
        Filter = Filter.ToUpper
        For Each result In srch.FindAll()
            de = result.GetDirectoryEntry

            buf = de.Properties("CN").Value.ToString
            If Filter = "" OrElse Filter = buf.ToUpper Or buf.ToUpper Like Filter Then
                If Svr_Only Then
                    If de.Properties("operatingSystem").Count > 0 Then
                        os = de.Properties("operatingSystem").Value.ToString
                        If os Like "*Server*" Then
                            sc.Add(buf)
                        End If
                    End If
                Else
                    sc.Add(buf)
                End If
            End If
        Next

        If sc.Count > 0 Then
            ReDim ans(sc.Count - 1)
            sc.CopyTo(ans, 0)
            [Array].Sort(ans)
        End If
        Return ans
    End Function

    '
    ' A recursive routine to generate a list of network resources that
    ' appear in the "My Network Places" browse list
    '   Level 0 = Provider (ie Microsoft Windows Network)
    '   Level 1 = Domain/Workgroup name
    '   Level 2 = PC names
    '   Level 3 = File/Print shares
    '
    Private Sub GetEnumNet(ByRef Net As NETRESOURCE, ByVal LevelLimit As Integer, ByVal StartingDomain As String, ByRef sc As StringCollection)
        Dim TempNet As New NETRESOURCE
        Dim ret, BufSize, NetCount, i As Integer
        Dim NetHandle, iPtr, iBuf As IntPtr
        Static Level As Integer
        Dim Win32Error As Win32Exception

        ' Is this the first time thru?
        If IsNothing(Net) Then
            If StartingDomain <> "" Then
                ' if we specified a starting Domain/Workgroup, then we create
                ' level 1 NETRESOURCE structure by hand, and start the search 
                ' at level 2.
                Level = 2
                TempNet.dwDisplayType = RESOURCEDISPLAYTYPE_DOMAIN
                TempNet.dwScope = RESOURCE_GLOBALNET
                TempNet.dwType = RESOURCETYPE_ANY
                TempNet.dwUsage = RESOURCEUSAGE_CONTAINER
                TempNet.lpProvider = "Microsoft Windows Network"
                TempNet.lpRemoteName = StartingDomain
                ret = WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_ANY, 0, TempNet, NetHandle)
            Else
                ' start at the top...
                Level = 0
                ret = WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_ANY, RESOURCEUSAGE_CONTAINER, Nothing, NetHandle)
            End If
        Else
            ret = WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_ANY, 0, Net, NetHandle)
        End If

        If ret <> 0 Then
            Win32Error = New Win32Exception(ret)
            Throw New ApplicationException(Win32Error.Message)
        End If

        ' Create 16k buffer to handle the results
        BufSize = 16384
        iBuf = Marshal.AllocHGlobal(BufSize)

        Do
            ' Query the handle returned by WNetOpenEnum
            NetCount = -1
            ret = WNetEnumResource(NetHandle, NetCount, iBuf, BufSize)
            If ret <> 0 And ret <> ERROR_NO_MORE_ITEMS Then
                ' silently ignore these errors
                Exit Do
            End If

            ' loop thru the answers
            For i = 0 To NetCount - 1
                ' some pointer wizardry to "walk" the NETRESOURCE array
                iPtr = New IntPtr(iBuf.ToInt64 + (Marshal.SizeOf(TempNet) * i))
                TempNet = CType(Marshal.PtrToStructure(iPtr, GetType(NETRESOURCE)), NETRESOURCE)

                ' copy the item name to our string collection
                If Level = LevelLimit Then
                    sc.Add(TempNet.lpRemoteName.Replace("\\", ""))
                End If

                ' make a recursive call to this subroutine
                If Level < LevelLimit Then
                    ' increment the level by 1
                    Level += 1
                    GetEnumNet(TempNet, LevelLimit, StartingDomain, sc)
                    Level -= 1
                End If
            Next
        Loop While ret <> ERROR_NO_MORE_ITEMS

        ' clean up and go home
        Marshal.FreeHGlobal(iBuf)
        WNetCloseEnum(NetHandle)
    End Sub
End Class
