Imports System.Text

'
' A class to create a text-based representation of the SOSOS database.  Used
' in both the Text view and in the saving/printing of a TextReport
' Note: Supports an optiona "DoDetails" flag that only affects Text-based
' reports/print job.  It does not affect the detail of data saved to the
' database.
'
Public Class TextReport

    Private ds As SOSOSDataSet

    Public Sub New(ByVal sosos_ds As SOSOSDataSet)
        ds = sosos_ds
    End Sub

    '
    ' Prepare the report based upon which item have been selected (via the
    ' individual "Do" settings and the DoDetails setting).
    '
    Public Function PrepareReport(ByVal doDetails As Boolean) As String
        Dim asm As System.Reflection.Assembly
        Dim sb As New StringBuilder

        ' let's see if we even have any data
        If ds.SOS.Count = 0 Then
            Return Nothing
        End If

        ' the report header
        asm = System.Reflection.Assembly.GetExecutingAssembly
        sb.AppendLine("Son of Snoop on Steroids (SOSOS) - Text Report")
        sb.AppendLine(asm.GetName.Name & " v" & asm.GetName.Version.ToString & " (" & asm.ImageRuntimeVersion & ")")

        ' load (but do not show) the about box, to get the following information
        Dim about As New fmAbout
        sb.AppendLine(about.lbAuthor.Text)
        sb.AppendLine("Email: " & about.llEmail.Text)
        sb.AppendLine("Home: " & about.llHome.Text)

        sb.AppendLine("Report Date: " & Now)
        sb.AppendLine("")

        ReportSection("SOS", sb)

        If My.Settings.DoAccounts Then
            sb.AppendLine("")
            ReportSection("Accounts", sb)
        End If

        If My.Settings.DoAdmins Then
            sb.AppendLine("")
            ReportSection("Admins", sb)
        End If

        If My.Settings.DoAutoUpdate Then
            sb.AppendLine("")
            ReportSection("AutoUpdate", sb)
        End If

        If My.Settings.DoBIOS Then
            sb.AppendLine("")
            ReportSection("BIOS", sb)
        End If

        If doDetails And My.Settings.DoComponents Then
            sb.AppendLine("")
            ReportSection("Components", sb)
        End If

        If My.Settings.DoCPU Then
            sb.AppendLine("")
            ReportSection("CPU", sb)
        End If

        If doDetails And My.Settings.DoDesktop Then
            sb.AppendLine("")
            ReportSection("Desktop", sb)
        End If

        If My.Settings.DoDevices Then
            sb.AppendLine("")
            ReportSection("Devices", sb)
        End If

        If My.Settings.DoDisks Then
            sb.AppendLine("")
            ReportSection("Disks", sb)
        End If

        If My.Settings.DoDrives Then
            sb.AppendLine("")
            ReportSection("Drives", sb)
        End If

        If doDetails And My.Settings.DoEmail Then
            sb.AppendLine("")
            ReportSection("Email", sb)
        End If

        If doDetails And My.Settings.DoEventLogs Then
            sb.AppendLine("")
            ReportSection("EventLogs", sb)
        End If

        If doDetails And My.Settings.DoEventLogSettings Then
            sb.AppendLine("")
            ReportSection("EventLogSettings", sb)
        End If

        If doDetails And My.Settings.DoFileInfo Then
            sb.AppendLine("")
            ReportSection("FileInfo", sb)
        End If

        If doDetails And My.Settings.DoInternet Then
            sb.AppendLine("")
            ReportSection("Internet", sb)
        End If

        If doDetails And My.Settings.DoMapped Then
            sb.AppendLine("")
            ReportSection("Mapped", sb)
        End If

        If My.Settings.DoMemory Then
            sb.AppendLine("")
            ReportSection("Memory", sb)
        End If

        If My.Settings.DoModem Then
            sb.AppendLine("")
            ReportSection("Modem", sb)
        End If

        If My.Settings.DoMonitor Then
            sb.AppendLine("")
            ReportSection("Monitor", sb)
        End If

        If My.Settings.DoMotherboard Then
            sb.AppendLine("")
            ReportSection("Motherboard", sb)
        End If

        If My.Settings.DoMouse Then
            sb.AppendLine("")
            ReportSection("Mouse", sb)
        End If

        If My.Settings.DoNetAdapter Then
            sb.AppendLine("")
            ReportSection("NetAdapter", sb)
        End If

        If My.Settings.DoNetConfig Then
            sb.AppendLine("")
            ReportSection("NetConfig", sb)
        End If

        If My.Settings.DoOS Then
            sb.AppendLine("")
            ReportSection("OS", sb)
        End If

        If My.Settings.DoPC Then
            sb.AppendLine("")
            ReportSection("PC", sb)
        End If

        If doDetails And My.Settings.DoPermissions Then
            sb.AppendLine("")
            ReportSection("Permissions", sb)
        End If

        If My.Settings.DoPorts Then
            sb.AppendLine("")
            ReportSection("Ports", sb)
        End If

        If My.Settings.DoPrinters Then
            sb.AppendLine("")
            ReportSection("Printers", sb)
        End If

        If doDetails And My.Settings.DoProcesses Then
            sb.AppendLine("")
            ReportSection("Processes", sb)
        End If

        If doDetails And My.Settings.DoProfiles Then
            sb.AppendLine("")
            ReportSection("Profiles", sb)
        End If

        If doDetails And My.Settings.DoQFE Then
            sb.AppendLine("")
            ReportSection("QFE", sb)
        End If

        If doDetails And My.Settings.DoRegistry Then
            sb.AppendLine("")
            ReportSection("Registry", sb)
        End If

        If doDetails And My.Settings.DoServices Then
            sb.AppendLine("")
            ReportSection("Services", sb)
        End If

        If My.Settings.DoShares Then
            sb.AppendLine("")
            ReportSection("Shares", sb)
        End If

        If doDetails And My.Settings.DoSoftware Then
            sb.AppendLine("")
            ReportSection("Software", sb)
        End If

        If doDetails And My.Settings.DoStartup Then
            sb.AppendLine("")
            ReportSection("Startup", sb)
        End If

        If doDetails And My.Settings.DoSystemDrivers Then
            sb.AppendLine("")
            ReportSection("SystemDrivers", sb)
        End If

        If My.Settings.DoSystemInfo Then
            sb.AppendLine("")
            ReportSection("SystemInfo", sb)
        End If

        If My.Settings.DoTasks Then
            sb.AppendLine("")
            ReportSection("Tasks", sb)
        End If

        If My.Settings.DoUserLogin Then
            sb.AppendLine("")
            ReportSection("UserLogin", sb)
        End If

        If My.Settings.DoVideo Then
            sb.AppendLine("")
            ReportSection("Video", sb)
        End If

        If doDetails And My.Settings.DoVirus Then
            sb.AppendLine("")
            ReportSection("Virus", sb)
        End If

        sb.AppendLine("")
        Return sb.ToString
    End Function

    '
    ' Build a section of the TextReport
    '
    Sub ReportSection(ByVal SectionName As String, ByVal sb As StringBuilder)
        Dim nodename As String = ""
        Dim dr As DataRow

        ' let's see if we even have any data
        If ds.SOS.Count = 0 Then
            Exit Sub
        End If

        ' Strongly-typed datasets have their place... but not when you need to
        ' convert a bunch of datarows with nulls to strings.  The NullValue 
        ' attributes in the DataSet are not configurable at runtime...
        Select Case SectionName
            Case "Accounts"
                sb.AppendLine("Accounts (Local Accounts)")
                For Each dr In ds.Accounts
                    sb.AppendLine(vbTab & "Name: " & dr("Account").ToString)
                    sb.AppendLine(vbTab & vbTab & "FullName: " & dr("Account_FullName").ToString)
                    sb.AppendLine(vbTab & vbTab & "Description: " & dr("Account_Description").ToString)
                    sb.AppendLine(vbTab & vbTab & "Disabled: " & dr("Account_Disabled").ToString)
                    sb.AppendLine(vbTab & vbTab & "LockedOut: " & dr("Account_LockedOut").ToString)
                    sb.AppendLine(vbTab & vbTab & "PwdRequired: " & dr("Account_PwdRequired").ToString)
                    sb.AppendLine(vbTab & vbTab & "PwdChangeable: " & dr("Account_PwdChangeable").ToString)
                    sb.AppendLine(vbTab & vbTab & "PwdExpires: " & dr("Account_PwdExpires").ToString)
                Next
            Case "Admins"
                sb.AppendLine("Admins (Administrators)")
                For Each dr In ds.Admins
                    sb.AppendLine(vbTab & "UserID: " & dr("Admin_UserID").ToString)
                    sb.AppendLine(vbTab & vbTab & "AcctType: " & dr("Admin_AcctType").ToString)
                Next
            Case "AutoUpdate"
                sb.AppendLine("AutoUpdate")
                For Each dr In ds.AutoUpdate
                    sb.AppendLine(vbTab & "LastQFE: " & dr("AutoUpdate_LastQFE").ToString)
                    sb.AppendLine(vbTab & "LastQFEDate: " & dr("AutoUpdate_LastQFEDate").ToString)
                    sb.AppendLine(vbTab & "Enabled: " & dr("AutoUpdate_Enabled").ToString)
                    sb.AppendLine(vbTab & "UseSUS: " & dr("AutoUpdate_UseSUS").ToString)
                    sb.AppendLine(vbTab & "Options: " & dr("AutoUpdate_Options").ToString)
                    sb.AppendLine(vbTab & "Server: " & dr("AutoUpdate_Server").ToString)
                    sb.AppendLine(vbTab & "State: " & dr("AutoUpdate_State").ToString)
                    sb.AppendLine(vbTab & "NextRun: " & dr("AutoUpdate_NextRun").ToString)
                Next
            Case "BIOS"
                sb.AppendLine("BIOS")
                For Each dr In ds.BIOS
                    sb.AppendLine(vbTab & "BIOS: " & dr("BIOS").ToString)
                    sb.AppendLine(vbTab & "Version: " & dr("BIOS_Version").ToString)
                    sb.AppendLine(vbTab & "Date: " & dr("BIOS_Date").ToString)
                Next
            Case "Components"
                sb.AppendLine("Components (Windows)")
                For Each dr In ds.Components
                    sb.AppendLine(vbTab & "Component: " & dr("Component").ToString)
                    sb.AppendLine(vbTab & vbTab & "Name: " & dr("Component_Name").ToString)
                    sb.AppendLine(vbTab & vbTab & "Version: " & dr("Component_Version").ToString)
                Next
            Case "CPU"
                sb.AppendLine("CPU")
                For Each dr In ds.CPU
                    sb.AppendLine(vbTab & "CPU: " & dr("CPU").ToString)
                    sb.AppendLine(vbTab & "Name: " & dr("CPU_Name").ToString)
                    sb.AppendLine(vbTab & "Socket: " & dr("CPU_Socket").ToString)
                    sb.AppendLine(vbTab & "Speed: " & dr("CPU_Speed").ToString)
                    sb.AppendLine(vbTab & "ExtSpeed: " & dr("CPU_ExtSpeed").ToString)
                    sb.AppendLine(vbTab & "Count: " & dr("CPU_Count").ToString)
                Next
            Case "Desktop"
                sb.AppendLine("Desktop Settings")
                For Each dr In ds.Desktop
                    sb.AppendLine(vbTab & "Profile: " & dr("ID_Profile").ToString)
                    sb.AppendLine(vbTab & vbTab & "Wallpaper: " & dr("Desktop_Wallpaper").ToString)
                    sb.AppendLine(vbTab & vbTab & "ScreenSaver: " & dr("Desktop_ScreenSaver").ToString)
                    sb.AppendLine(vbTab & vbTab & "SS_Active: " & dr("Desktop_SS_Active").ToString)
                    sb.AppendLine(vbTab & vbTab & "SS_Timeout: " & dr("Desktop_SS_Timeout").ToString)
                    sb.AppendLine(vbTab & vbTab & "SS_IsSecure: " & dr("Desktop_SS_IsSecure").ToString)
                Next
            Case "Devices"
                sb.AppendLine("Devices")
                ' group the listings by Category
                For Each dr In ds.Devices
                    If nodename <> dr("Device_Category").ToString Then
                        nodename = dr("Device_Category").ToString
                        sb.AppendLine(vbTab & nodename)
                    End If
                    sb.AppendLine(vbTab & vbTab & "Device: " & dr("Device").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Driver: " & dr("Device_Driver").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Status: " & dr("Device_Status").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "PnPDeviceID: " & dr("Device_PnPDeviceID").ToString)
                Next
            Case "Disks"
                sb.AppendLine("Disks (Physical Disks)")
                For Each dr In ds.Disks
                    sb.AppendLine(vbTab & "Disk: " & dr("Disk_Desc").ToString)
                    sb.AppendLine(vbTab & vbTab & "Interface: " & dr("Disk_Interface").ToString)
                    sb.AppendLine(vbTab & vbTab & "Partitions: " & dr("Disk_Partitions").ToString)
                    sb.AppendLine(vbTab & vbTab & "SerialNumber: " & dr("Disk_SerialNumber").ToString)
                Next
            Case "Drives"
                sb.AppendLine("Drives (Drive Letters)")
                For Each dr In ds.Drives
                    sb.AppendLine(vbTab & "Drive: " & dr("Drive").ToString)
                    sb.AppendLine(vbTab & vbTab & "Size: " & dr("Drive_Size").ToString)
                    sb.AppendLine(vbTab & vbTab & "FreeSpace: " & dr("Drive_FreeSpace").ToString)
                    sb.AppendLine(vbTab & vbTab & "Type: " & dr("Drive_Type").ToString)
                    sb.AppendLine(vbTab & vbTab & "FileSystem: " & dr("Drive_FileSystem").ToString)
                    sb.AppendLine(vbTab & vbTab & "MediaType: " & dr("Drive_MediaType").ToString)
                    sb.AppendLine(vbTab & vbTab & "VolumeName: " & dr("Drive_VolumeName").ToString)
                Next
            Case "Email"
                sb.AppendLine("Email Settings")
                For Each dr In ds.Email
                    sb.AppendLine(vbTab & "Profile: " & dr("ID_Profile").ToString)
                    sb.AppendLine(vbTab & vbTab & "ProfileName: " & dr("Email_ProfileName").ToString)
                    sb.AppendLine(vbTab & vbTab & "PST_Path: " & dr("Email_PST_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "PST_Size: " & dr("Email_PST_Size").ToString)
                    sb.AppendLine(vbTab & vbTab & "CachedMode: " & dr("Email_CachedMode").ToString)
                    sb.AppendLine(vbTab & vbTab & "ExchangeServer: " & dr("Email_ExchangeServer").ToString)
                Next
            Case "EventLogs"
                sb.AppendLine("EventLogs")
                For Each dr In ds.EventLogs
                    ' group the listings by Logfile
                    If nodename <> dr("Event_Logfile").ToString Then
                        nodename = dr("Event_Logfile").ToString
                        sb.AppendLine(vbTab & nodename)
                    End If
                    sb.AppendLine(vbTab & vbTab & "ID: " & dr("Event_ID").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Type: " & dr("Event_Type").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Date: " & dr("Event_Date").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Source: " & dr("Event_Source").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Code: " & dr("Event_Code").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "User: " & dr("Event_User").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Message: " & dr("Event_Message").ToString)
                Next
            Case "EventLogSettings"
                sb.AppendLine("EventLogSettings")
                For Each dr In ds.EventLogSettings
                    sb.AppendLine(vbTab & "Logfile: " & dr("EventSettings_Logfile").ToString)
                    sb.AppendLine(vbTab & vbTab & "CurrentSize: " & dr("EventSettings_CurrentSize").ToString)
                    sb.AppendLine(vbTab & vbTab & "MaxSize: " & dr("EventSettings_MaxSize").ToString)
                    sb.AppendLine(vbTab & vbTab & "Retension: " & dr("EventSettings_Retension").ToString)
                    sb.AppendLine(vbTab & vbTab & "NumEntries: " & dr("EventSettings_NumEntries").ToString)
                    sb.AppendLine(vbTab & vbTab & "OldestEntry: " & dr("EventSettings_OldestEntry").ToString)
                Next
            Case "FileInfo"
                sb.AppendLine("FileInfo")
                For Each dr In ds.FileInfo
                    sb.AppendLine(vbTab & "Path: " & dr("FileInfo_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "Version: " & dr("FileInfo_Version").ToString)
                    sb.AppendLine(vbTab & vbTab & "Size: " & dr("FileInfo_Size").ToString)
                    sb.AppendLine(vbTab & vbTab & "ModDate: " & dr("FileInfo_ModDate").ToString)
                Next
            Case "Internet"
                sb.AppendLine("Internet Settings")
                For Each dr In ds.Internet
                    sb.AppendLine(vbTab & "Profile: " & dr("ID_Profile").ToString)
                    sb.AppendLine(vbTab & vbTab & "CacheSize: " & dr("Internet_CacheSize").ToString)
                    sb.AppendLine(vbTab & vbTab & "ProxyEnabled: " & dr("Internet_ProxyEnabled").ToString)
                    sb.AppendLine(vbTab & vbTab & "ProxyServers: " & dr("Internet_ProxyServers").ToString)
                    sb.AppendLine(vbTab & vbTab & "ProxyExceptions: " & dr("Internet_ProxyExceptions").ToString)
                    sb.AppendLine(vbTab & vbTab & "ProxyBypassLocal: " & dr("Internet_ProxyBypassLocal").ToString)
                Next
            Case "Mapped"
                sb.AppendLine("Mapped (Network Drives)")
                For Each dr In ds.Mapped
                    sb.AppendLine(vbTab & "Profile: " & dr("ID_Profile").ToString)
                    sb.AppendLine(vbTab & vbTab & "Mapped: " & dr("Mapped").ToString)
                    sb.AppendLine(vbTab & vbTab & "Path: " & dr("Mapped_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "Type: " & dr("Mapped_Type").ToString)
                    sb.AppendLine(vbTab & vbTab & "Provider: " & dr("Mapped_Provider").ToString)
                Next
            Case "Memory"
                sb.AppendLine("Memory (RAM)")
                For Each dr In ds.Memory
                    sb.AppendLine(vbTab & "Memory: " & dr("Memory").ToString)
                    sb.AppendLine(vbTab & "Type: " & dr("Memory_Type").ToString)
                    sb.AppendLine(vbTab & "Form: " & dr("Memory_Form").ToString)
                    sb.AppendLine(vbTab & "Count: " & dr("Memory_Count").ToString)
                    sb.AppendLine(vbTab & "Width: " & dr("Memory_Width").ToString)
                    sb.AppendLine(vbTab & "Speed: " & dr("Memory_Speed").ToString)
                Next
            Case "Modem"
                sb.AppendLine("Modem")
                For Each dr In ds.Modem
                    sb.AppendLine(vbTab & "ID: " & dr("Modem_ID").ToString)
                    sb.AppendLine(vbTab & vbTab & "Modem: " & dr("Modem").ToString)
                    sb.AppendLine(vbTab & vbTab & "Port: " & dr("Modem_Port").ToString)
                    sb.AppendLine(vbTab & vbTab & "Type: " & dr("Modem_Type").ToString)
                Next
            Case "Monitor"
                sb.AppendLine("Monitor (Video)")
                For Each dr In ds.Monitor
                    sb.AppendLine(vbTab & "Monitor: " & dr("Monitor").ToString)
                    sb.AppendLine(vbTab & vbTab & "Size: " & dr("Monitor_Size").ToString)
                Next
            Case "Motherboard"
                sb.AppendLine("Motherboard")
                For Each dr In ds.Motherboard
                    sb.AppendLine(vbTab & "Motherboard: " & dr("Motherboard").ToString)
                    sb.AppendLine(vbTab & "Model: " & dr("Motherboard_Model").ToString)
                    sb.AppendLine(vbTab & "SerialNumber: " & dr("Motherboard_SerialNumber").ToString)
                    sb.AppendLine(vbTab & "Version: " & dr("Motherboard_Version").ToString)
                Next
            Case "Mouse"
                sb.AppendLine("Mouse")
                For Each dr In ds.Mouse
                    sb.AppendLine(vbTab & "Mouse: " & dr("Mouse").ToString)
                    sb.AppendLine(vbTab & vbTab & "Type: " & dr("Mouse_Type").ToString)
                    sb.AppendLine(vbTab & vbTab & "Buttons: " & dr("Mouse_Buttons").ToString)
                    sb.AppendLine(vbTab & vbTab & "Port: " & dr("Mouse_Port").ToString)
                Next
            Case "NetAdapter"
                sb.AppendLine("NetAdapter (Network Card)")
                For Each dr In ds.NetAdapter
                    sb.AppendLine(vbTab & "NetAdapter: " & dr("NetAdapter").ToString)
                    sb.AppendLine(vbTab & vbTab & "Manufacturer: " & dr("NetAdapter_Manufacturer").ToString)
                    sb.AppendLine(vbTab & vbTab & "MAC_Address: " & dr("NetAdapter_MAC").ToString)
                Next
            Case "NetConfig"
                sb.AppendLine("NetConfig (IP Configuration)")
                For Each dr In ds.NetConfig
                    sb.AppendLine(vbTab & "Adapter: " & dr("NetConfig").ToString)
                    sb.AppendLine(vbTab & vbTab & "DHCP: " & dr("NetConfig_DHCP").ToString)
                    sb.AppendLine(vbTab & vbTab & "IP: " & dr("NetConfig_IP").ToString)
                    sb.AppendLine(vbTab & vbTab & "Subnet: " & dr("NetConfig_Subnet").ToString)
                    sb.AppendLine(vbTab & vbTab & "Router: " & dr("NetConfig_Router").ToString)
                    sb.AppendLine(vbTab & vbTab & "DNSName: " & dr("NetConfig_DNSName").ToString)
                    sb.AppendLine(vbTab & vbTab & "DNSDomain: " & dr("NetConfig_DNSDomain").ToString)
                    sb.AppendLine(vbTab & vbTab & "DNS1: " & dr("NetConfig_DNS1").ToString)
                    sb.AppendLine(vbTab & vbTab & "DNS2: " & dr("NetConfig_DNS2").ToString)
                    sb.AppendLine(vbTab & vbTab & "WINS1: " & dr("NetConfig_WINS1").ToString)
                    sb.AppendLine(vbTab & vbTab & "WINS2: " & dr("NetConfig_WINS2").ToString)
                Next
            Case "OS"
                sb.AppendLine("OS")
                For Each dr In ds.OS
                    sb.AppendLine(vbTab & "OS: " & dr("OS").ToString)
                    sb.AppendLine(vbTab & "Version: " & dr("OS_Version").ToString)
                    sb.AppendLine(vbTab & "InstallDate: " & dr("OS_InstallDate").ToString)
                    sb.AppendLine(vbTab & "RegUser: " & dr("OS_RegUser").ToString)
                    sb.AppendLine(vbTab & "Organization: " & dr("OS_Organization").ToString)
                    sb.AppendLine(vbTab & "SerialNumber: " & dr("OS_SerialNumber").ToString)
                Next
            Case "PC"
                sb.AppendLine("PC (Configuration)")
                For Each dr In ds.PC
                    sb.AppendLine(vbTab & "PC: " & dr("PC").ToString)
                    sb.AppendLine(vbTab & "Comment: " & dr("PC_Comment").ToString)
                    sb.AppendLine(vbTab & "ChassisType: " & dr("PC_ChassisType").ToString)
                    sb.AppendLine(vbTab & "Domain: " & dr("PC_DomainName").ToString)
                    sb.AppendLine(vbTab & "NetBIOS Domain: " & dr("PC_DomainNameNB").ToString)
                    sb.AppendLine(vbTab & "Path: " & dr("PC_ADPath").ToString)
                    sb.AppendLine(vbTab & "SID: " & dr("PC_SID").ToString)
                    sb.AppendLine(vbTab & "Role: " & dr("PC_DomainRole").ToString)
                    sb.AppendLine(vbTab & "NumUsers: " & dr("PC_NumUsers").ToString)
                    sb.AppendLine(vbTab & "Timezone: " & dr("PC_Timezone").ToString)
                    sb.AppendLine(vbTab & "DST: " & dr("PC_DSTEnabled").ToString)
                    sb.AppendLine(vbTab & "LastBoot: " & dr("PC_LastBoot").ToString)
                    sb.AppendLine(vbTab & "LastLogin: " & dr("PC_LastLogin").ToString)
                    sb.AppendLine(vbTab & "NetServices: " & dr("PC_NetServices").ToString)
                    sb.AppendLine(vbTab & "HAL: " & dr("PC_HAL").ToString)
                    sb.AppendLine(vbTab & "JoinedDomain: " & dr("PC_JoinedDomain").ToString)
                    sb.AppendLine(vbTab & "SystemType: " & dr("PC_SystemType").ToString)
                Next
            Case "Permissions"
                sb.AppendLine("Permissions")
                For Each dr In ds.Permissions
                    ' group the listings by Target
                    If nodename <> dr("Perm_Target").ToString Then
                        nodename = dr("Perm_Target").ToString
                        sb.AppendLine(vbTab & nodename)
                    End If
                    sb.AppendLine(vbTab & vbTab & "ID: " & dr("Perm_ID").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Type: " & dr("Perm_Type").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Name: " & dr("Perm_Name").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Permission: " & dr("Perm_Permission").ToString)
                Next
            Case "Ports"
                sb.AppendLine("Ports (Serial, Parallel, etc)")
                For Each dr In ds.Ports
                    sb.AppendLine(vbTab & "Serial: " & dr("Ports_Serial").ToString)
                    sb.AppendLine(vbTab & "Parallel: " & dr("Ports_Parallel").ToString)
                    sb.AppendLine(vbTab & "IR: " & dr("Ports_IR").ToString)
                    sb.AppendLine(vbTab & "PCMCIA: " & dr("Ports_PCMCIA").ToString)
                    sb.AppendLine(vbTab & "USB: " & dr("Ports_USB").ToString)
                    sb.AppendLine(vbTab & "Firewire: " & dr("Ports_Firewire").ToString)
                Next
            Case "Printers"
                sb.AppendLine("Printers")
                For Each dr In ds.Printers
                    sb.AppendLine(vbTab & "Printer: " & dr("Printer").ToString)
                    sb.AppendLine(vbTab & vbTab & "Port: " & dr("Printer_Port").ToString)
                    sb.AppendLine(vbTab & vbTab & "SharedAs: " & dr("Printer_SharedAs").ToString)
                Next
            Case "Processes"
                sb.AppendLine("Processes")
                For Each dr In ds.Processes
                    sb.AppendLine(vbTab & "ProcessID: " & dr("ProcessID").ToString)
                    sb.AppendLine(vbTab & vbTab & "Process " & dr("Process").ToString)
                    sb.AppendLine(vbTab & vbTab & "Path: " & dr("Process_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "Owner: " & dr("Process_Owner").ToString)
                    sb.AppendLine(vbTab & vbTab & "Memory: " & dr("Process_Memory").ToString)
                Next
            Case "Profiles"
                sb.AppendLine("User Profiles")
                For Each dr In ds.Profiles
                    sb.AppendLine(vbTab & "Profile: " & dr("Profile").ToString)
                    sb.AppendLine(vbTab & vbTab & "Path: " & dr("Profile_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "Owner: " & dr("Profile_Owner").ToString)
                    sb.AppendLine(vbTab & vbTab & "IsAdmin: " & dr("Profile_IsAdmin").ToString)
                    sb.AppendLine(vbTab & vbTab & "LastLoad: " & dr("Profile_LastLoad").ToString)
                Next
            Case "QFE"
                sb.AppendLine("QFE (Updates)")
                For Each dr In ds.QFE
                    sb.AppendLine(vbTab & "QFE: " & dr("QFE").ToString)
                    sb.AppendLine(vbTab & vbTab & "Category: " & dr("QFE_Category").ToString)
                    sb.AppendLine(vbTab & vbTab & "Description: " & dr("QFE_Description").ToString)
                    sb.AppendLine(vbTab & vbTab & "InstallDate: " & dr("QFE_InstallDate").ToString)
                Next
            Case "Registry"
                sb.AppendLine("Registry Values")
                For Each dr In ds.Registry
                    sb.AppendLine(vbTab & "Key: " & dr("Registry_Key").ToString)
                    sb.AppendLine(vbTab & vbTab & "Value: " & dr("Registry_Value").ToString)
                Next
            Case "Services"
                sb.AppendLine("Services")
                For Each dr In ds.Services
                    sb.AppendLine(vbTab & "Service: " & dr("Service").ToString)
                    sb.AppendLine(vbTab & vbTab & "Path: " & dr("Service_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "Status: " & dr("Service_Status").ToString)
                    sb.AppendLine(vbTab & vbTab & "StartType: " & dr("Service_StartType").ToString)
                    sb.AppendLine(vbTab & vbTab & "AcctName: " & dr("Service_AcctName").ToString)
                Next
            Case "Shares"
                sb.AppendLine("Shares")
                For Each dr In ds.Shares
                    sb.AppendLine(vbTab & "Share: " & dr("Share_Desc").ToString)
                    sb.AppendLine(vbTab & vbTab & "Path: " & dr("Share_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "Type: " & dr("Share_Type").ToString)
                Next
            Case "Software"
                sb.AppendLine("Software (Installed Software)")
                For Each dr In ds.Software
                    sb.AppendLine(vbTab & "Software: " & dr("Software").ToString)
                    sb.AppendLine(vbTab & vbTab & "Vendor: " & dr("Software_Vendor").ToString)
                    sb.AppendLine(vbTab & vbTab & "Version: " & dr("Software_Version").ToString)
                    sb.AppendLine(vbTab & vbTab & "InstallDate: " & dr("Software_InstallDate").ToString)
                    sb.AppendLine(vbTab & vbTab & "InstallSrc: " & dr("Software_InstallSrc").ToString)
                Next
            Case "SOS"
                dr = ds.SOS(0)
                sb.AppendLine("SOS")
                sb.AppendLine(vbTab & "Name: " & dr("PC_Name").ToString)
                sb.AppendLine(vbTab & "Domain: " & dr("PC_Domain").ToString)
                sb.AppendLine(vbTab & "Model: " & dr("PC_Model").ToString)
                sb.AppendLine(vbTab & "User: " & dr("UserName").ToString)
                sb.AppendLine(vbTab & "IsAdmin: " & dr("IsAdmin").ToString)
                sb.AppendLine(vbTab & "RecordType: " & dr("RecordType").ToString)
                sb.AppendLine(vbTab & "Comment: " & dr("PC_Comment").ToString)
                sb.AppendLine(vbTab & "Date: " & dr("CurrentDate").ToString)
                sb.AppendLine(vbTab & "IP_Address: " & dr("IP_Address").ToString)
                sb.AppendLine(vbTab & "MAC_Address: " & dr("MAC_Address").ToString)
                sb.AppendLine(vbTab & "NetCard: " & dr("NetCard").ToString)
                sb.AppendLine(vbTab & "DHCP: " & dr("DHCP").ToString)
                sb.AppendLine(vbTab & "Net_Services: " & dr("Net_Services").ToString)
                sb.AppendLine(vbTab & "OS: " & dr("OS").ToString)
                sb.AppendLine(vbTab & "OS_Build: " & dr("OS_Build").ToString)
                sb.AppendLine(vbTab & "CPU_Type: " & dr("CPU_Type").ToString)
                sb.AppendLine(vbTab & "CPU_Speed: " & dr("CPU_Speed").ToString)
                sb.AppendLine(vbTab & "CPU_Count: " & dr("CPU_Count").ToString)
                sb.AppendLine(vbTab & "Memory: " & dr("Memory").ToString)
                sb.AppendLine(vbTab & "Disk_Size: " & dr("Disk_Size").ToString)
                sb.AppendLine(vbTab & "Disk_Free: " & dr("Disk_Free").ToString)
                sb.AppendLine(vbTab & "Disk_Count: " & dr("Disk_Count").ToString)
                sb.AppendLine(vbTab & "CDROM: " & dr("CDROM").ToString)
                sb.AppendLine(vbTab & "SoundCard: " & dr("SoundCard").ToString)
                sb.AppendLine(vbTab & "Modem: " & dr("Modem").ToString)
                sb.AppendLine(vbTab & "PCMCIA: " & dr("PCMCIA").ToString)
                sb.AppendLine(vbTab & "SCSI: " & dr("SCSI").ToString)
                sb.AppendLine(vbTab & "USB: " & dr("USB").ToString)
                sb.AppendLine(vbTab & "SmartCard: " & dr("SmartCard").ToString)
                sb.AppendLine(vbTab & "Printer: " & dr("Printer").ToString)
                sb.AppendLine(vbTab & "Install_Rev: " & dr("Install_Rev").ToString)
                sb.AppendLine(vbTab & "Install_Date: " & dr("Install_Date").ToString)
                sb.AppendLine(vbTab & "MSOffice_Version: " & dr("MSOffice_Version").ToString)
                sb.AppendLine(vbTab & "IE_Version: " & dr("IE_Version").ToString)
                sb.AppendLine(vbTab & "Virus_Version: " & dr("Virus_Version").ToString)
                sb.AppendLine(vbTab & "Virus_Defs: " & dr("Virus_Defs").ToString)
                sb.AppendLine(vbTab & "Virus_Scan: " & dr("Virus_Scan").ToString)
                sb.AppendLine(vbTab & "AutoUpdate: " & dr("AutoUpdate").ToString)
            Case "Startup"
                sb.AppendLine("Startup")
                For Each dr In ds.Startup
                    ' group the listings by Profile
                    If nodename <> dr("ID_Profile").ToString Then
                        nodename = dr("ID_Profile").ToString
                        sb.AppendLine(vbTab & "Profile: " & nodename)
                    End If
                    sb.AppendLine(vbTab & vbTab & "Name: " & dr("Startup_Name").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Location: " & dr("Startup_Location").ToString)
                    sb.AppendLine(vbTab & vbTab & vbTab & "Path: " & dr("Startup_Path").ToString)
                Next
            Case "SystemDrivers"
                sb.AppendLine("System Drivers")
                For Each dr In ds.SystemDrivers
                    sb.AppendLine(vbTab & "Name: " & dr("Driver_Name").ToString)
                    sb.AppendLine(vbTab & vbTab & "Description: " & dr("Driver_Description").ToString)
                    sb.AppendLine(vbTab & vbTab & "Path: " & dr("Driver_Path").ToString)
                    sb.AppendLine(vbTab & vbTab & "Type: " & dr("Driver_Type").ToString)
                    sb.AppendLine(vbTab & vbTab & "StartMode: " & dr("Driver_StartMode").ToString)
                Next
            Case "SystemInfo"
                sb.AppendLine("System (Hardware)")
                For Each dr In ds.SystemInfo
                    sb.AppendLine(vbTab & "Manufacturer: " & dr("System_Manufacturer").ToString)
                    sb.AppendLine(vbTab & "Model: " & dr("System_Model").ToString)
                    sb.AppendLine(vbTab & "SerialNumber: " & dr("System_SerialNumber").ToString)
                    sb.AppendLine(vbTab & "UUID: " & dr("System_UUID").ToString)
                Next
            Case "Tasks"
                sb.AppendLine("Scheduled Tasks")
                For Each dr In ds.Tasks
                    sb.AppendLine(vbTab & "JobName: " & dr("Task_JobName").ToString)
                    sb.AppendLine(vbTab & vbTab & "State: " & dr("Task_State").ToString)
                    sb.AppendLine(vbTab & vbTab & "Comment: " & dr("Task_Comment").ToString)
                    sb.AppendLine(vbTab & vbTab & "Command: " & dr("Task_Command").ToString)
                    sb.AppendLine(vbTab & vbTab & "Schedule: " & dr("Task_Schedule").ToString)
                    sb.AppendLine(vbTab & vbTab & "LastRun: " & dr("Task_LastRun").ToString)
                    sb.AppendLine(vbTab & vbTab & "Status: " & dr("Task_Status").ToString)
                    sb.AppendLine(vbTab & vbTab & "LastResult: " & dr("Task_LastResult").ToString)
                    sb.AppendLine(vbTab & vbTab & "Account: " & dr("Task_Account").ToString)
                Next
            Case "UserLogin"
                sb.AppendLine("UserLogin")
                For Each dr In ds.UserLogin
                    sb.AppendLine(vbTab & "UserID: " & dr("UserID").ToString)
                    sb.AppendLine(vbTab & "LoginTime: " & dr("LoginTime").ToString)
                    sb.AppendLine(vbTab & "AcctDomain: " & dr("AcctDomain").ToString)
                    sb.AppendLine(vbTab & "AcctName: " & dr("AcctName").ToString)
                    sb.AppendLine(vbTab & "ResourceDomain: " & dr("ResourceDomain").ToString)
                    sb.AppendLine(vbTab & "PCName: " & dr("PCName").ToString)
                Next
            Case "Video"
                sb.AppendLine("Video (Video Card)")
                For Each dr In ds.Video
                    sb.AppendLine(vbTab & "Video: " & dr("Video").ToString)
                    sb.AppendLine(vbTab & vbTab & "Frequency: " & dr("Video_Frequency").ToString)
                    sb.AppendLine(vbTab & vbTab & "ScanMode: " & dr("Video_ScanMode").ToString)
                    sb.AppendLine(vbTab & vbTab & "Driver: " & dr("Video_Driver").ToString)
                    sb.AppendLine(vbTab & vbTab & "AdapterRAM: " & dr("Video_AdapterRAM").ToString)
                    sb.AppendLine(vbTab & vbTab & "Resolution: " & dr("Video_Resolution").ToString)
                    sb.AppendLine(vbTab & vbTab & "ColorDepth: " & dr("Video_ColorDepth").ToString)
                Next
            Case "Virus"
                sb.AppendLine("Virus (Virus Activity)")
                For Each dr In ds.Virus
                    sb.AppendLine(vbTab & "ID: " & dr("Virus_ID").ToString)
                    sb.AppendLine(vbTab & vbTab & "Name: " & dr("Virus_Name").ToString)
                    sb.AppendLine(vbTab & vbTab & "Date: " & dr("Virus_Date").ToString)
                    sb.AppendLine(vbTab & vbTab & "User: " & dr("Virus_User").ToString)
                    sb.AppendLine(vbTab & vbTab & "Location: " & dr("Virus_Location").ToString)
                    sb.AppendLine(vbTab & vbTab & "Action: " & dr("Virus_Action").ToString)
                Next
        End Select
    End Sub
End Class
